/*
 * Copyright (C) 2012, 2013 Canonical, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; version 3.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

import QtQuick 2.0
import Ubuntu.Components 0.1

Item {
    id: appStack

    property var model: undefined

    signal selectedApplication(string selectedApplication, string type)

    ListModel {
        id: internalModel
    }

    property int selectedIndex: -1
    property bool completed: false

    Component.onCompleted: {
        completed = true
        resetSelected()
    }

    onModelChanged: {
        if (completed) {
            resetSelected()
        }
    }

    Connections {
        target: model
        // Accumulate count changes since hud clears
        // and then readds items to the models which means 
        // even if the result is the same we get lots of count changes
        onCountChanged: mergeModelsTimer.restart()
    }

    Timer {
        id: mergeModelsTimer
        interval: 30
        onTriggered: mergeModels()
    }

    function setSideStageSelected() {
        for (var i = 0; i < internalModel.count; ++i) {
            var data = internalModel.get(i)
            if (data.appType == 1) {
                setSelectedIndex(i, true)
                break
            }
        }
    }

    function mergeModels() {
        // Merge models
        var i = 0
        var selectedApp = selectedIndex >= 0 && selectedIndex < internalModel.count ? internalModel.get(selectedIndex).appId :  ""
        // Find which is the first item that does not match
        for ( ; i < internalModel.count && i < model.count; ++i) {
            if (internalModel.get(i).appId != model.get(i).column_0 ||
                internalModel.get(i).appIcon != "image://gicon/" + model.get(i).column_1 ||
                internalModel.get(i).appType != model.get(i).column_2)
            {
                break;
            }
        }
        // Remove the rest of items after the last one that matches in the internal model
        if (i < internalModel.count) {
            internalModel.remove(i, internalModel.count - i)
        }
        var firstAdded = -1
        repeater.itemsAnimationEnabled = false
        // Add all the different items to the internalModel with scale of 0.8
        for ( ; i < model.count; ++i) {
            var itemData = model.get(i)
            internalModel.append({"appId": itemData.column_0, "appIcon": "image://gicon/" + itemData.column_1, "appType": itemData.column_2, "overrideScale": 0.8})
            if (firstAdded == -1)
                firstAdded = i
        }
        repeater.itemsAnimationEnabled = true
        if (firstAdded != -1) {
            // Start the scale 0.8 -> 1 animation
            var o = internalModel.get(firstAdded)
            o.overrideScale = -1
        }

        // If the first added is the first item, select it, since it is
        // the focused app position, meaning the focused app changed
        // and we want it to be the selected item in the appstack
        if (firstAdded == 0) {
            setSelectedIndex(0, true)
        } else {
            // See if the old selected app is still in the model
            for (i = 0 ; i < internalModel.count; ++i) {
                if (internalModel.get(i).appId == selectedApp) {
                    // Are indices the same?
                    if (i != selectedIndex) {
                        // No call to setSelectedIndex since we don't need to either
                        // select and unselect the old app nor emit the signal
                        // since the app hasn't actually changed, just the index
                        selectedIndex = i
                        item(selectedIndex).selected = true
                    }
                    break;
                }
            }
            if (i == internalModel.count) {
                // The old selected app is no longer in the model
                setSelectedIndex((model.count > 0) ? 0 : -1, true)
            }
        }
    }

    function resetSelected() {
        internalModel.clear()
        for (var i = 0; i < model.count; ++i) {
            var itemData = model.get(i)
            internalModel.append({"appId": itemData.column_0, "appIcon": "image://gicon/" + itemData.column_1, "appType": itemData.column_2, "overrideScale": -1})
        }
        setSelectedIndex((model.count > 0) ? 0 : -1, true)
    }

    function setSelectedIndex(newIndex, modelContentsChanged) {
        if (selectedIndex != newIndex || modelContentsChanged) {
            var oldSelectedItem = item(selectedIndex)
            if (oldSelectedItem)
                oldSelectedItem.selected = false
            var newSelectedItem = item(newIndex)
            if (newSelectedItem)
                newSelectedItem.selected = true
            selectedIndex = newIndex
            if (selectedIndex >= 0 && selectedIndex < internalModel.count) {
                var data = internalModel.get(selectedIndex)
                var type = "mainStageApplication"
                if (data.appType == 1) type = "sideStageApplication"
                else if (data.appType == 2) type = "backgroundApplication"
                else if (data.appType == 3) type = "indicator"
                selectedApplication(data.appId, type)
            } else {
                selectedApplication("", "none")
            }
        }
        flickable.calculateCropping()
    }

    function item(index) {
        if (index < 0 || index >= model.count)
            return undefined

        return appStackRow.children[index]
    }

    clip: true
    height: specialItem.height

    AppStackItem {
        id: specialItem
        objectName: "appStackSpecialItem"

        property bool atLeft: true

        icon: ""
        visible: icon != ""
        anchors.verticalCenter: parent.verticalCenter
        anchors.left: atLeft ? parent.left : undefined
        anchors.right: anchors.left == parent.left ? undefined : parent.right
        selected: true
    }

    Item {
        id: flickableCropper
        readonly property double specialTileWidth: specialItem.width + units.gu(1)
        x: specialItem.visible && specialItem.atLeft ? specialTileWidth : 0
        anchors.verticalCenter: parent.verticalCenter
        width: parent.width - specialTileWidth
        height: flickable.height
        clip: specialItem.visible

        Flickable {
            x: -parent.x
            id: flickable
            objectName: "appStackFlickable"
            contentWidth: appStackRow.width
            contentHeight: appStackRow.height
            height: contentHeight
            width: appStack.width
            anchors.verticalCenter: parent.verticalCenter
            onContentXChanged: calculateCropping()

            function calculateCropping() {
                var selectedItem = item(selectedIndex)
                if (!selectedItem)
                    return;

                if (-contentX + selectedItem.x < 0) {
                    specialItem.atLeft = true
                    specialItem.icon = selectedItem.icon
                } else if (-contentX + selectedItem.x + selectedItem.width >= appStack.width) {
                    specialItem.atLeft = false
                    specialItem.icon = selectedItem.icon
                } else if (specialItem.visible) {
                    specialItem.icon = ""
                }
            }

            Timer {
                id: itemShowTimer
                property variant nextItem
                interval: 50
                onTriggered: nextItem.scale = 1
            }

            Row {
                id: appStackRow
                objectName: "appStackRow"
                spacing: units.gu(0.5)
                Repeater {
                    id: repeater
                    property bool itemsAnimationEnabled: true
                    model: internalModel
                    delegate: AppStackItem {
                        icon: appIcon
                        visible: scale > 0.8
                        Behavior on scale {
                            enabled: repeater.itemsAnimationEnabled
                            NumberAnimation {
                                duration: 250
                                onRunningChanged: {
                                    if (running) {
                                        var nextItem = item(index + 1)
                                        if (nextItem != undefined) {
                                            itemShowTimer.nextItem = nextItem
                                            itemShowTimer.start()
                                        }
                                    }
                                }
                            }
                        }
                        MouseArea {
                            anchors.fill: parent
                            onClicked: setSelectedIndex(index, false)
                        }
                    }
                }
            }
        }
    }
}
