/*
 * Copyright (C) 2013 Canonical, Ltd.
 *
 * Authors:
 *  Michał Sawicz <michal.sawicz@canonical.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; version 3.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

// self
#include "categories.h"

// Utils
#include "plugins/Utils/qsortfilterproxymodelqml.h"

class CategoryFilter : public QSortFilterProxyModelQML
{
    Q_OBJECT

    Q_PROPERTY(int index READ index WRITE setIndex NOTIFY indexChanged)

public:
    explicit CategoryFilter(QObject* parent = 0)
        : QSortFilterProxyModelQML(parent)
        , m_index(-1)
    {
        setDynamicSortFilter(true);
        setFilterRole(2); // category index column
        setFilterRegExp(QString("^%1$").arg(m_index));
    }

    /* getters */
    int index() const { return m_index; }

    /* setters */
    void setIndex(int index)
    {
        if (index != m_index) {
            m_index = index;
            setFilterRegExp(QString("^%1$").arg(m_index));
            Q_EMIT indexChanged(m_index);
        }
    }

Q_SIGNALS:
    void indexChanged(int index);

private:
    int m_index;
};


Categories::Categories(QObject* parent)
    : DeeListModel(parent)
    , m_resultModel(0)
    , m_globalResultModel(0)
{
    // FIXME: need to clean up unused filters on countChanged
    m_roles[Categories::RoleId] = "id";
    m_roles[Categories::RoleName] = "name";
    m_roles[Categories::RoleIcon] = "icon";
    m_roles[Categories::RoleRenderer] = "renderer";
    m_roles[Categories::RoleHints] = "hints";
    m_roles[Categories::RoleResults] = "results";
    m_roles[Categories::RoleGlobalResults] = "globalResults";
    m_roles[Categories::RoleCount] = "count";
    m_roles[Categories::RoleGlobalCount] = "globalCount";

    // TODO This should not be needed but accumulatting the count changes
    // makes the visualization more stable and also makes crashes on fast
    // change of the search term harder to reproduce
    m_timer.setSingleShot(true);
    m_timer.setInterval(50);
    connect(&m_timer, SIGNAL(timeout()), this, SLOT(onEmitCountChanged()));
}

Categories::~Categories()
{
    qDeleteAll(m_filters);
    qDeleteAll(m_globalFilters);
}

CategoryFilter*
Categories::getFilter(int index) const
{
    if (!m_filters.contains(index)) {
        CategoryFilter* filter = new CategoryFilter();
        connect(filter, SIGNAL(countChanged()), this, SLOT(onCountChanged()));
        filter->setSourceModelQObject(m_resultModel);
        filter->setIndex(index);

        m_filters.insert(index, filter);
    }

    return m_filters[index];
}

CategoryFilter*
Categories::getGlobalFilter(int index) const
{
    if (!m_globalFilters.contains(index)) {
        CategoryFilter* filter = new CategoryFilter();
        connect(filter, SIGNAL(countChanged()), this, SLOT(onGlobalCountChanged()));
        filter->setSourceModelQObject(m_globalResultModel);
        filter->setIndex(index);

        m_globalFilters.insert(index, filter);
    }

    return m_globalFilters[index];
}

void
Categories::setResultModel(DeeListModel* model)
{
    if (model != m_resultModel) {
        m_resultModel = model;

        Q_FOREACH(CategoryFilter* filter, m_filters) {
            filter->setSourceModelQObject(m_resultModel);
        }

        Q_EMIT resultModelChanged(m_resultModel);
    }
}

void
Categories::setGlobalResultModel(DeeListModel* model)
{
    if (model != m_globalResultModel) {
        m_globalResultModel = model;

        Q_FOREACH(CategoryFilter* filter, m_globalFilters) {
            filter->setSourceModelQObject(m_globalResultModel);
        }

        Q_EMIT globalResultModelChanged(m_globalResultModel);
    }
}

void
Categories::onCountChanged()
{
    CategoryFilter* filter = qobject_cast<CategoryFilter*>(sender());
    if (filter) {
        m_timerFilters << filter;
        m_timer.start();
    }
}

void
Categories::onEmitCountChanged()
{
    QVector<int> roles;
    roles.append(Categories::RoleCount);
    Q_FOREACH(CategoryFilter* filter, m_timerFilters) {
        QModelIndex changedIndex = index(filter->index());		
        Q_EMIT dataChanged(changedIndex, changedIndex, roles);
    }
    m_timerFilters.clear();
}

void
Categories::onGlobalCountChanged()
{
    CategoryFilter* filter = qobject_cast<CategoryFilter*>(sender());
    if (filter) {
        QModelIndex changedIndex = index(filter->index());
        QVector<int> roles;
        roles.append(Categories::RoleGlobalCount);
        Q_EMIT dataChanged(changedIndex, changedIndex, roles);
    }
}

QHash<int, QByteArray>
Categories::roleNames() const
{
    return m_roles;
}

QVariant
Categories::data(const QModelIndex& index, int role) const
{
    if (!index.isValid()) {
        return QVariant();
    }

    if (role == RoleId) {
        return QVariant::fromValue(index.row());
    } else if (role == RoleName) {
        return QVariant::fromValue(DeeListModel::data(index, 0));
    } else if (role == RoleIcon) {
        return QVariant::fromValue(DeeListModel::data(index, 1));
    } else if (role == RoleRenderer) {
        return QVariant::fromValue(DeeListModel::data(index, 2));
    } else if (role == RoleHints) {
        return QVariant::fromValue(DeeListModel::data(index, 3));
    } else if (role == RoleResults) {
        // FIXME: the cast shouldn't be necessary
        return QVariant::fromValue(qobject_cast<QSortFilterProxyModelQML*>(getFilter(index.row())));
    } else if (role == RoleGlobalResults) {
        // FIXME: ditto
        return QVariant::fromValue(qobject_cast<QSortFilterProxyModelQML*>(getGlobalFilter(index.row())));
    } else if (role == RoleCount) {
        CategoryFilter* filter = getFilter(index.row());
        return QVariant::fromValue(filter->rowCount());
    } else if (role == RoleGlobalCount) {
        QSortFilterProxyModel* filter = getGlobalFilter(index.row());
        return QVariant::fromValue(filter->rowCount());
    } else {
        return QVariant();
    }
}

#include "categories.moc"
