/*
 * Copyright 2013 Canonical Ltd.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; version 3.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

import QtQuick 2.0
import QtTest 1.0
import HudClient 0.1

import "../../Hud"

Hud {
    id: hud
    height: units.gu(80)
    width: units.gu(50)

    // Mock shell to shut up some warnings
    property QtObject shell: QtObject {
        property QtObject applicationManager: QtObject {
            id: applicationManager
            property bool keyboardVisible: false
        }
    }

    HudClientStub {
        id: hudClientStub
    }

    UnityTestCase {
        name: "HudTest"
        when: windowShown

        function resetToInitialState() {
            hud.show()
            hud.resetToInitialState()
            hudClientStub.reset()
        }

        function test_hud_toolbar() {
            var toolBar = findChild(hud, "toolBar")
            resetToInitialState()

            compare(toolBar.children.length, 5, "toolbar item count")

            mouseClick(toolBar.children[0], 0, 0)
            compare(hudClientStub.lastExecutedToolbarItem, hudClientStub.quitToolbarItemValue(), "Click on quit")

            mouseClick(toolBar.children[1], 0, 0)
            compare(hudClientStub.lastExecutedToolbarItem, hudClientStub.undoToolbarItemValue(), "Click on undo")

            mouseClick(toolBar.children[2], 0, 0)
            compare(hudClientStub.lastExecutedToolbarItem, hudClientStub.helpToolbarItemValue(), "Click on help")

            mouseClick(toolBar.children[3], 0, 0)
            compare(hudClientStub.lastExecutedToolbarItem, hudClientStub.fullScreenToolbarItemValue(), "Click on fullscreen")

            mouseClick(toolBar.children[4], 0, 0)
            compare(hudClientStub.lastExecutedToolbarItem, hudClientStub.preferencesToolbarItemValue(), "Click on preferences")
        }

        function test_hud_appstack_static_contents() {
            var appStack = findChild(hud, "appStack")
            resetToInitialState()

            compare(appStack.model.count, 2, "appstack item count")

            var firstApp = appStack.model.get(0)
            compare(firstApp["column_0"], "app0", "first appstack item name")
            compare(firstApp["column_1"], "help", "first appstack item icon")
            compare(firstApp["column_2"], 0, "first appstack item type")

            var secondApp = appStack.model.get(1)
            compare(secondApp["column_0"], "app1", "second appstack item name")
            compare(secondApp["column_1"], "folder", "second appstack item icon")
            compare(secondApp["column_2"], 1, "second appstack item type")
        }

        function test_hud_appstack_click_app() {
            var appStackRow = findChild(hud, "appStackRow")
            resetToInitialState()

            mouseClick(appStackRow.children[1], 0, 0)
            compare(hudClientStub.lastSetAppStackApplicationId, "app1", "Click on second item")
        }

        function test_hud_appstack_special_item() {
            var appStackFlickable = findChild(hud, "appStackFlickable")
            var appStackSpecialItem = findChild(hud, "appStackSpecialItem")
            var appStackRow = findChild(hud, "appStackRow")
            var appStack = findChild(hud, "appStack")
            resetToInitialState()

            // Wait until the flickable is properly layouted
            tryCompare(appStackRow.children[1], "x", appStackRow.children[0].width + units.gu(0.5))
            compare(appStackSpecialItem.visible, false, "Special item not visible at start")

            mouseClick(appStackRow.children[0], 0, 0)

            // Check that when dragging left the special item appears at pos 0
            var start_x = units.gu(12)
            var stop_x = units.gu(5)
            mouseFlick(appStackFlickable, start_x, 0, stop_x, 0, true, false)
            tryCompare(appStackSpecialItem, "visible", true)
            compare(appStackSpecialItem.x, 0, "Special item on left")
            mouseRelease(appStackFlickable, stop_x, 0)
            tryCompare(appStackSpecialItem, "visible", false)

            // Check that when dragging right the special item appears at pos appStack.width - itemwidth
            start_x = units.gu(1)
            stop_x = units.gu(150)
            mouseFlick(appStackFlickable, start_x, 0, stop_x, 0, true, false)
            tryCompare(appStackSpecialItem, "visible", true)
            compare(appStackSpecialItem.x + appStackSpecialItem.width, appStack.width, "Special item on right")
            mouseRelease(appStackFlickable, stop_x, 0)
            tryCompare(appStackSpecialItem, "visible", false)
        }

        function test_hud_results() {
            var resultListRepeater = findChild(hud, "resultListRepeater")
            resetToInitialState()

            tryCompare(resultListRepeater, "count", 5)

            compare(resultListRepeater.itemAt(0).children[1].nameText, "Help", "First result name")
            compare(resultListRepeater.itemAt(0).children[1].contextSnippetText, "Get Help", "First result description")

            compare(resultListRepeater.itemAt(1).children[1].nameText, "About", "Second result name")
            compare(resultListRepeater.itemAt(1).children[1].contextSnippetText, "Show About", "Second result description")

            compare(resultListRepeater.itemAt(2).children[1].nameText, "Foo", "Third result name")
            compare(resultListRepeater.itemAt(2).children[1].contextSnippetText, "Show Foo", "Third result description")

            compare(resultListRepeater.itemAt(3).children[1].nameText, "Bar", "Fourth result name")
            compare(resultListRepeater.itemAt(3).children[1].contextSnippetText, "Show Bar", "Fourth result description")

            compare(resultListRepeater.itemAt(4).children[1].nameText, "FooBar", "Fifth result name")
            compare(resultListRepeater.itemAt(4).children[1].contextSnippetText, "Show FooBar", "Fifth result description")
        }

        function test_hud_set_query() {
            var searchBar = findChild(hud, "searchBar")
            resetToInitialState()

            compare(hudClientStub.lastSetQuery, "", "Query should be empty at start")

            searchBar.text = "Hi"
            compare(hudClientStub.lastSetQuery, "Hi", "Query should be updated")

            searchBar.text = "Ho"
            compare(hudClientStub.lastSetQuery, "Ho", "Query should be updated")
        }

        function compareItemAnchoredAtTopOf(itemA, itemB, spacing, text) {
            compare(itemA.y - spacing, itemB.y, text)
        }

        function compareItemAnchoredAtBottomOf(itemA, itemB, spacing, text) {
            compare(itemA.y + itemA.height + spacing, itemB.y + itemB.height, text)
        }

        function compareItemAnchoredUnderItem(itemUnder, itemTop, spacing, text) {
            compare(itemUnder.y - spacing, itemTop.y + itemTop.height, text)
        }

        function compareItemAnchoredOverItem(itemTop, itemUnder, spacing, text) {
            compareItemAnchoredUnderItem(itemUnder, itemTop, spacing, text)
        }

        function test_hud_elements_positioning() {
            var searchBar = findChild(hud, "searchBar")
            var searchBarContainer = findChild(hud, "searchBarAnimator")
            var appStackContainer = findChild(hud, "appStackAnimator")
            var resultListContainer = findChild(hud, "resultsCardAnimator")
            var toolBarContainer = findChild(hud, "toolBarAnimator")
            var toolBar = findChild(hud, "toolBar")
            var handle = findChild(hud, "handle")
            resetToInitialState()

            compare(toolBar.visible, true, "Toolbar is initially visible")
            compareItemAnchoredAtTopOf(handle, hud, 0, "Handle is anchored at the top")
            compareItemAnchoredAtBottomOf(searchBarContainer, hud, hud.elementsPadding, "SearchBar is anchored at the bottom")
            compareItemAnchoredUnderItem(appStackContainer, handle, hud.elementsPadding + units.dp(1), "AppStack is anchored under the handle")
            compareItemAnchoredOverItem(toolBarContainer, searchBarContainer, 2 * hud.elementsPadding, "Toolbar is anchored over the searchBar")
            compareItemAnchoredOverItem(resultListContainer, toolBarContainer, hud.elementsPadding, "ResultsList is anchored over the toolBar")

            mouseClick(searchBar, 0, 0)

            tryCompare(toolBar, "visible", false)
            compareItemAnchoredAtTopOf(handle, hud, 0, "Handle is anchored at the top")
            compareItemAnchoredUnderItem(appStackContainer, handle, hud.elementsPadding + units.dp(1), "AppStack is anchored under the handle")
            compareItemAnchoredUnderItem(searchBarContainer, appStackContainer, hud.elementsPadding, "SearchBar is anchored under the appStack")
            compareItemAnchoredUnderItem(resultListContainer, searchBarContainer, hud.elementsPadding, "ResultsList is anchored under the searchBar")
        }

        function test_hud_result_suggestion_execution() {
            var resultListRepeater = findChild(hud, "resultListRepeater")
            resetToInitialState()

            mouseClick(resultListRepeater.itemAt(1), 0, 0)
            compare(hudClientStub.lastExecutedCommandRow, 1, "Last executed row was 1")
        }
    }
}
